@@ -387,6 +387,9 @@ bool AAptInterface::startAptGetInstall()
 	iAptGetCurrentFileTotalSize = 0;
 	iAptGetCurrentDownloadFileName = "";
 	iProgressCheckTimer = 0;
+	iSpeedKbps = 0;
+	iSpeedKbpsPrev = -1;
+	iUpdateSpeed = false;
 
 	bool useproxy = iSettings->qsettings()->value("use_proxies",false).toBool();
 	QString http_proxy = iSettings->qsettings()->value("http_proxy","").toString();
@@ -836,8 +839,6 @@ void AAptInterface::uiUpdaterAptGetInstall()
 			}
 		}
 	}
-
-
 }
 
 void AAptInterface::progressCheckTimerCallback()
@@ -845,14 +846,29 @@ void AAptInterface::progressCheckTimerCallback()
 	if( iAptGetCurrentDownloadFileName=="" )
 		return;
 
+	qint64 prevsize = iAptGetCurrentFileDownloadSize;
 	QFile pkgfile(KAptArchivePartialDir + "/" + iAptGetCurrentDownloadFileName);
 	iAptGetCurrentFileDownloadSize = pkgfile.size()/1024;
 
+	if( iAptGetCurrentFileDownloadSize >= prevsize ) {
+		iSpeedKbpsPrev = iSpeedKbps;
+		iSpeedKbps = (iAptGetCurrentFileDownloadSize-prevsize)*2;
+	} else {
+		iSpeedKbpsPrev = -1;
+	}
+	if( iUpdateSpeed )
+		iUpdateSpeed = false;
+	else
+		iUpdateSpeed = true;
+
 	if( iUiDimmer && iUiDimmer->busy() ) {
 		int p = iAptGetCurrentFileDownloadSize*100/iAptGetCurrentFileTotalSize;
 		if( iAptGetDownloadCount > 0 && iAptGetCurrentFileDownloadSize==0 )
 			p = 100;
 		iUiDimmer->setProgress( p );
+		if( iSpeedKbps>=0 && iSpeedKbpsPrev>=0 && iUpdateSpeed ) {
+			iUiDimmer->setDownloadSpeed( (iSpeedKbps+iSpeedKbpsPrev)/2 );
+		}
 	}
 }
 
@@ -1359,11 +1375,27 @@ bool AAptInterface::processPackageDataLine(Package*& pkg, QByteArray& line)
 	}
 	else if( line.startsWith("Depends: ") )
 	{
-		pkg->setDepends( line.mid(9) );
+		pkg->appendDepends( line.mid(9) );
 	}
 	else if( line.startsWith("Conflicts: ") )
 	{
-		pkg->setConflicts( line.mid(11) );
+		pkg->appendConflicts( line.mid(11) );
+	}
+	else if( line.startsWith("Pre-Depends: ") )
+	{
+		pkg->appendPreDepends( line.mid(13) );
+	}
+	else if( line.startsWith("Provides: ") )
+	{
+		pkg->appendProvides( line.mid(10) );
+	}
+	else if( line.startsWith("Replaces: ") )
+	{
+		pkg->appendReplaces( line.mid(10) );
+	}
+	else if( line.startsWith("Breaks: ") )
+	{
+		pkg->appendBreaks( line.mid(8) );
 	}
 
 	if( iMultiLine == MultiLineDesc ) {
@@ -1479,7 +1511,7 @@ void AAptInterface::readBlacklist()
 		Package* pkg = iPackagesAvailable.value(parts.at(0).trimmed(), 0);
 		Package* pkg2 = iPackagesInstalled.value(parts.at(0).trimmed(), 0);
 		if( parts.count()==1 ) {
-			if( pkg && !pkg2 )
+			if( pkg )
 				pkg->setBlacklisted(BlacklistSelect::BlacklistAll);
 			if( pkg2 )
 				pkg2->setBlacklisted(BlacklistSelect::BlacklistAll);
@@ -1586,7 +1618,9 @@ void AAptInterface::startFetchDates()
 	iDateRequestsReceived = 0;
 	iNetworkError = QNetworkReply::NoError;
 
-	qDebug() << "package dates to fetch:" << fetchable.count();
+	QString dbgcount = QString("need to fetch date for %1 packages").arg(fetchable.count());
+	qDebug() << dbgcount;
+	logToFile(dbgcount);
 
 	QHashIterator<QString, Package*> fe( fetchable );
 	while (fe.hasNext() && !iTerminated)
@@ -1637,6 +1671,7 @@ void AAptInterface::startFetchDates()
 					iNetworkError != QNetworkReply::QNetworkReply::ContentAccessDenied && iNetworkError != QNetworkReply::QNetworkReply::ContentNotFoundError )
 				{
 					qDebug() << "fatal network error, aborting fetch";
+					logToFile(QString("Fatal network error, date fetch aborted"));
 					break;
 				} else {
 					nam->head(r);
@@ -1680,8 +1715,10 @@ void AAptInterface::startFetchDates()
 		QApplication::processEvents();
 	}
 
-	qDebug() << "sent" << iDateRequestsSent << "requests, received" << iDateRequestsReceived << "replies with"
-			<< iDateRequestErrors << "errors";
+	QString dbgstr = QString("sent %1 requests, received %2 replies with %3 errors").arg(iDateRequestsSent).arg(iDateRequestsReceived).arg(iDateRequestErrors);
+
+	qDebug() << dbgstr;
+	logToFile(dbgstr);
 
 	logToFile( QString("Finished fetching package dates") );
 
@@ -1726,7 +1763,9 @@ void AAptInterface::dateFetchNetworkReply(QNetworkReply* reply)
 	}
 
 	if( reply->error() != QNetworkReply::NoError ) {
-		qDebug() << reply->error() << reply->errorString();
+		QString dbg = reply->url().fragment() + QString(": error %1: ").arg(reply->error()) + reply->errorString();
+		qDebug() << dbg;
+		logToFile( dbg );
 		iDateRequestErrors++;
 	}
 	reply->deleteLater();
@@ -1750,6 +1789,7 @@ void AAptInterface::writeDateCache()
 		f.close();
 	} else {
 		qDebug() << "Warning: failed to write date cache";
+		logToFile(QString("Failed to write date cache"));
 	}
 }
 
@@ -1772,10 +1812,12 @@ void AAptInterface::readDateCache()
 						pkg->setDate( dt );
 					} else {
 						qDebug() << "Warning: Invalid date in date cache";
+						logToFile(QString("Invalid date in date cache"));
 					}
 				}
 			} else {
 				qDebug() << "Warning: error in date cache:" << line;
+				logToFile(QString("Error in date cache"));
 			}
 		}
 		f.close();
