@@ -179,10 +179,13 @@ private:
 	int iAptGetRemoveCount;
 	int iAptGetInstallTotal;
 	int iAptGetRemoveTotal;
-	int iAptGetCurrentFileDownloadSize;
-	int iAptGetCurrentFileTotalSize;
+	qint64 iAptGetCurrentFileDownloadSize;
+	qint64 iAptGetCurrentFileTotalSize;
 	QString iAptGetCurrentDownloadFileName;
 	QTimer* iProgressCheckTimer;
+	int iSpeedKbps;
+	int iSpeedKbpsPrev;
+	bool iUpdateSpeed;
 
 	int iCatalogCounter;
 	int iCatalogsTotal;
