@@ -35,6 +35,7 @@ public:
 	bool busy() { return iBusy; }
 	void updateText(QString message);
 	void setProgress(int p_);
+	void setDownloadSpeed(int kbps_);
 	int progress() { return iProgress; }
 
 private:
@@ -52,6 +53,7 @@ private:
 	bool iUpdateAnim;
 
 	int iProgress;
+	int iDownloadSpeed;
 
 public slots:
 	void orientationChanged();
