@@ -68,12 +68,11 @@ MainWindow::MainWindow(QWidget *parent) :
 
 	iWinPackageView->setSortOrder( (PackageView::sortOrder)iSettings->qsettings()->value("default_sort_order",0).toInt() );
 
-	iWinPackageView->setSearchOptions( iSettings->qsettings()->value("search_pkgnames",false).toBool(),
+	iWinPackageView->setSearchOptions( iSettings->qsettings()->value("search_pkgnames",true).toBool(),
 									   iSettings->qsettings()->value("search_displaynames",true).toBool(),
-									   iSettings->qsettings()->value("search_descshort",false).toBool(),
+									   iSettings->qsettings()->value("search_descshort",true).toBool(),
 									   iSettings->qsettings()->value("search_desclong",false).toBool() );
 
-
 #ifdef Q_WS_MAEMO_5
 	this->setAttribute(Qt::WA_Maemo5StackedWindow);
 	if( !iSettings->qsettings()->value("disable_autorotation",false).toBool() ) {
@@ -368,6 +367,7 @@ void MainWindow::operationQueueFinished(QList<AAptInterface::interfaceMode> last
 		}
 		pkglist += "</font>";
 
+		bool installing_blacklisted = false;
 		if( inst.count()>0 ) {
 			pkglist += "<br><b><u>INSTALL/UPGRADE:</u></b><br><font size=\"-1\">";
 			for( int i=0; i<inst.count(); i++ ) {
@@ -378,6 +378,11 @@ void MainWindow::operationQueueFinished(QList<AAptInterface::interfaceMode> last
 					pkglist += "<font color=\"red\">***NEW/UNKNOWN***</font>";
 					mismatch = true;
 				}
+				if( pkg && pkg->isBlacklisted() ) {
+					qDebug() << "Warning: installing blacklisted package" << inst.at(i);
+					pkglist += "<font color=\"red\">***BLACKLISTED***</font>";
+					installing_blacklisted = true;
+				}
 				if( instver.count()>i ) {
 					pkglist += " " + instver.at(i);
 					if( pkg && instver.at(i) != pkg->version() ) {
@@ -400,6 +405,11 @@ void MainWindow::operationQueueFinished(QList<AAptInterface::interfaceMode> last
 					  "from the repositories. This could be due to your application catalogs being out of date.");
 			m.exec();
 		}
+		if( installing_blacklisted ) {
+			ConfirmDialog b(false, this);
+			b.setText("Warning","Blacklisted package(s) will be installed");
+			b.exec();
+		}
 
 		busyDialog(false);
 		ConfirmDialog d(true, this);
