@@ -22,6 +22,7 @@
 
 #include <QtCore>
 #include <QMainWindow>
+#include <phonon/MediaObject>
 #include "aaptinterface.h"
 
 class PackageView;
@@ -30,6 +31,7 @@ class Dialog;
 class dimmer;
 class RepoView;
 class Settings;
+class DpkgInterface;
 
 namespace Ui {
     class MainWindow;
@@ -54,6 +56,7 @@ public:
 	void notifyDialog(QString title, QString msg);
 	bool confirmDialog(QString title, QString msg);
 	void resetIdlingTime() { iIdlingSince = QDateTime::currentDateTime(); }
+	void setNextOperation(nextOperation op_) { iNextOperation=op_; }
 
 public slots:
 	void on_btnUpdate_clicked();
@@ -69,6 +72,7 @@ private:
 
     Ui::MainWindow *ui;
 	AAptInterface* iAptInterface;
+	DpkgInterface* iDpkgInterface;
 	PackageView* iWinPackageView;
 	RepoView* iWinRepoView;
 	Dialog* iDlgBusy;
@@ -78,11 +82,13 @@ private:
 	bool iReposAutoUpdating;
 	bool iUpgradeAutoUpdate;
 	nextOperation iNextOperation;
+	Phonon::MediaObject* iMediaObject;
 
 	QDateTime iIdlingSince;
 
 private slots:
-	void on_actionOptions_triggered();
+	void on_actionLoad_file_triggered();
+ void on_actionOptions_triggered();
  void on_actionView_log_triggered();
 	void on_btnRepos_clicked();
 	void on_actionClean_triggered();
