@@ -52,8 +52,12 @@ public:
 	void addRepository(Repository* r_) { iRepositories.append(r_); }
 	void setDate(QDateTime d_) { iDate=d_; }
 	void setBlacklisted(BlacklistSelect::blackList bl_) { iBlacklist=bl_; }
-	void setDepends(QByteArray depends_);
-	void setConflicts(QByteArray conflicts_);
+	void appendDepends(QByteArray line_);
+	void appendConflicts(QByteArray line_);
+	void appendPreDepends(QByteArray line_);
+	void appendProvides(QByteArray line_);
+	void appendReplaces(QByteArray line_);
+	void appendBreaks(QByteArray line_);
 	void appendUpgradeDescription(QByteArray d_) { iUpgradeDescription.append(d_); }
 
 	void convertIcon();
@@ -82,8 +86,12 @@ public:
 	QList<Repository*> repositories() { return iRepositories; }
 	BlacklistSelect::blackList blacklisted() { return iBlacklist; }
 	bool isBlacklisted() { if(iBlacklist==BlacklistSelect::BlacklistNone) return false; else return true; }
-	QStringList depends();
-	QStringList conflicts();
+	QStringList dependsRichText();
+	QStringList conflictsRichText();
+	QStringList preDependsRichText();
+	QStringList providesRichText();
+	QStringList replacesRichText();
+	QStringList breaksRichText();
 	QString upgradeDescription() { return QString::fromUtf8(iUpgradeDescription); }
 
 	static bool versionCompare(QString isNewer, QString compare);
@@ -114,6 +122,10 @@ private:
 
 	QList<QByteArray> iDepends;
 	QList<QByteArray> iConflicts;
+	QList<QByteArray> iPreDepends;
+	QList<QByteArray> iProvides;
+	QList<QByteArray> iReplaces;
+	QList<QByteArray> iBreaks;
 };
 
 #endif // PACKAGE_H
