@@ -36,6 +36,9 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
     ui->setupUi(this);	
 	iPkg = pkg;
 	iAptInterface = apt;
+	iNetworkAccessManager = 0;
+	iChangelogFetched = false;
+	iChangelog = "";
 
 	ui->pushButton_website->setIcon(QPixmap("/usr/share/icons/hicolor/48x48/hildon/general_web.png"));
 	if( getMaemoOrgUrl(pkg)=="" && getMaemoOrgUrl(pkg->availablePackage())=="" ) {
@@ -59,17 +62,18 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 	ui->radioTabBasic->setChecked(true);
 	updateInfo();
 
-	// shorter button labels for portait mode
+	// switch to shorter button labels for portait mode
 	if( QApplication::desktop()->width() < QApplication::desktop()->height() )
 	{
 		ui->radioTabBasic->setText("Info");
-		ui->radioTabChanges->setText("Changes");
+		ui->radioTabChanges->setText("Chgs");
 		ui->radioTabDepends->setText("Deps");
 	}
 
 	Package* upg_pkg = iPkg->availablePackage();
-	//if( (!pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="")) && getMaemoOrgUrl(iPkg)=="" )
-	if( (!pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="")) )
+	if( !upg_pkg )
+		upg_pkg = iPkg;
+	if( (!pkg->isUpgradeable() || (upg_pkg && upg_pkg->upgradeDescription()=="")) && (getMaemoOrgUrl(upg_pkg)=="") )
 		ui->radioTabChanges->hide();
 
 	if( pkg->isUpgradeable() )
@@ -101,6 +105,8 @@ PackageSelector::PackageSelector(Package* pkg, AAptInterface* apt, QWidget *pare
 
 PackageSelector::~PackageSelector()
 {
+	delete iNetworkAccessManager;
+	iNetworkAccessManager = 0;
     delete ui;
 }
 
@@ -132,14 +138,40 @@ void PackageSelector::on_pushButton_blacklist_clicked()
 	/*if( iPkg->isUpgradeable() && iPkg->availablePackage() )
 		pkg = iPkg->availablePackage();*/
 
+	if( pkg->isInstalled() && pkg->isUpgradeable() )
+	{
+		Package* upg = pkg->availablePackage();
+		if( upg )
+			pkg = upg;
+	}
+
 	BlacklistSelect s(pkg, this);
 	BlacklistSelect::blackList old = pkg->blacklisted();
-	if( s.exec() ) {
-		updateInfo();
+	if( s.exec() ) {		
 		if( old != pkg->blacklisted() ) {
-			iAptInterface->removeFromBlacklist(pkg, old);
+			Package* p1 = iAptInterface->packagesInstalled()->value(pkg->name(),0);
+			Package* p2 = iAptInterface->packagesAvailable()->value(pkg->name(),0);
+			if( p1 ) {
+				iAptInterface->removeFromBlacklist(p1,old);
+				if( pkg->blacklisted()==BlacklistSelect::BlacklistAll || pkg->blacklisted()==BlacklistSelect::BlacklistNone )
+					p1->setBlacklisted( pkg->blacklisted() );
+				else if( pkg->blacklisted()==BlacklistSelect::BlacklistThis && pkg->version()==p1->version() )
+					p1->setBlacklisted( pkg->blacklisted() );
+				else
+					p1->setBlacklisted( BlacklistSelect::BlacklistNone );
+			}
+			if( p2 ) {
+				iAptInterface->removeFromBlacklist(p2,old);
+				if( pkg->blacklisted()==BlacklistSelect::BlacklistAll || pkg->blacklisted()==BlacklistSelect::BlacklistNone )
+					p2->setBlacklisted( pkg->blacklisted() );
+				else if( pkg->blacklisted()==BlacklistSelect::BlacklistThis && pkg->version()==p2->version() )
+					p2->setBlacklisted( pkg->blacklisted() );
+				else
+					p2->setBlacklisted( BlacklistSelect::BlacklistNone );
+			}
 			iAptInterface->writeBlacklist();
 		}
+		updateInfo();
 	}
 }
 
@@ -169,6 +201,30 @@ void PackageSelector::updateInfo()
 		on_radioTabChanges_clicked();
 	else if( ui->radioTabDepends->isChecked() )
 		on_radioTabDepends_clicked();
+
+	ui->radioInstall->setEnabled(true);
+	ui->radioRemove->setEnabled(true);
+	ui->radioInstall->show();
+	ui->radioRemove->show();
+
+	Package* upg_pkg = iPkg->availablePackage();
+
+	if( iPkg->isInstalled() && !iPkg->isUpgradeable() )
+	{
+		ui->radioInstall->setEnabled(false);
+		ui->radioInstall->hide();
+	} else if( !iPkg->isInstalled() ) {
+		ui->radioRemove->setEnabled(false);
+		ui->radioRemove->hide();
+	}
+
+	if( iPkg->isBlacklisted() ) {
+		ui->radioInstall->setEnabled(false);
+		//ui->radioRemove->setEnabled(false);
+	}
+	if( iPkg->isInstalled() && upg_pkg && upg_pkg->isBlacklisted() ) {
+		ui->radioInstall->setEnabled(false);
+	}
 }
 
 void PackageSelector::on_pushButton_website_clicked()
@@ -195,6 +251,8 @@ void PackageSelector::on_radioTabBasic_clicked()
 	ui->label_header2->show();
 	QString header2;
 
+	Package* upg_pkg = iPkg->availablePackage();
+
 	if( iPkg->isInstalled() ) {
 		header2 += "Installed, version <b>" + iPkg->version() + "</b>";
 	} else {
@@ -213,7 +271,7 @@ void PackageSelector::on_radioTabBasic_clicked()
 		header2 += "Marked for <b>removal</b><br>";
 	}
 
-	if( iPkg->isBlacklisted() ) {
+	if( iPkg->isBlacklisted() && !iPkg->isUpgradeable() ) {
 		header2 += "BLACKLISTED";
 		if( iPkg->blacklisted() == BlacklistSelect::BlacklistAll )
 			header2 += " (all)";
@@ -221,6 +279,15 @@ void PackageSelector::on_radioTabBasic_clicked()
 			header2 += " (this)";
 		iPkg->setMarkedForOperation(Package::PkgOpNone);
 		header2 += "<br>";
+	} else if( upg_pkg && upg_pkg->isBlacklisted() ) {
+		header2 += "BLACKLISTED";
+		if( upg_pkg->blacklisted() == BlacklistSelect::BlacklistAll )
+			header2 += " (all)";
+		else if( upg_pkg->blacklisted() == BlacklistSelect::BlacklistThis )
+			header2 += " (" + upg_pkg->version() + ")";
+		iPkg->setMarkedForOperation(Package::PkgOpNone);
+		upg_pkg->setMarkedForOperation(Package::PkgOpNone);
+		header2 += "<br>";
 	}
 
 	ui->label_header2->setText(header2);
@@ -292,7 +359,6 @@ void PackageSelector::on_radioTabBasic_clicked()
 	if( iPkg->isUpgradeable() )
 	{
 		longtext += "Repositories: ";
-		Package* upg_pkg = iPkg->availablePackage();
 		if( upg_pkg && upg_pkg->repositories().count()>0 ) {
 			for(int i=0; i<upg_pkg->repositories().count(); i++ ) {
 				if( upg_pkg->repositories().at(i) )
@@ -316,64 +382,42 @@ void PackageSelector::on_radioTabBasic_clicked()
 	}
 
 	ui->label_text->setText( longtext );
-
-	ui->radioInstall->setEnabled(true);
-	ui->radioRemove->setEnabled(true);
-	ui->radioInstall->show();
-	ui->radioRemove->show();
-
-	if( iPkg->isInstalled() && !iPkg->isUpgradeable() )
-	{
-		ui->radioInstall->setEnabled(false);
-		ui->radioInstall->hide();
-	} else if( !iPkg->isInstalled() ) {
-		ui->radioRemove->setEnabled(false);
-		ui->radioRemove->hide();
-	}
-
-	if( iPkg->isBlacklisted() ) {
-		ui->radioInstall->setEnabled(false);
-		ui->radioRemove->setEnabled(false);
-	}
 }
 
 void PackageSelector::on_radioTabChanges_clicked()
 {
-	/*
-	ui->label_header2->show();
-	ui->label_text->setText("");
-	ui->label_header2->setText("Fetching changelog...");
-	QApplication::processEvents();
+	ui->label_header2->hide();
+	ui->label_header2->setText("");
+
+	QString text;
 
-	QString changelog;
-	if( getMaemoOrgUrl(iPkg)!="" )
-		 changelog = queryChangelog();
-	*/
 	Package* upg_pkg = iPkg->availablePackage();
 	if( !upg_pkg )
 		upg_pkg = iPkg;
 
-	QString text;
 	if( iPkg->isUpgradeable() && upg_pkg && upg_pkg->upgradeDescription()!="" )
 	{
-		text += "Upgrade description:";
-		text += "<font size=\"-1\"><br><br>";
+		text += "<u><b>Upgrade description:</b></u>";
+		text += "<font size=\"-1\"><br>";
 		text += upg_pkg->upgradeDescription();
-		text += "</font><br><br>";
+		text += "</font><br>";
 		text.replace('\n',"<br>");
 	}
 
-	/*
-	text += "Debian changelog:<font size=\"-1\"><br><br>";
-	if( changelog != "" )
-		text += changelog;
-	else
-		text += "Not available";
+	QString changelog;
+	if( getMaemoOrgUrl(upg_pkg)!="" && !iChangelogFetched ) {
+		queryChangelog();
+		changelog = "Fetching changelog...";
+	} else {
+		changelog = iChangelog;
+		if( changelog == "" )
+			changelog = "Not available";
+	}
+
+	text += "<u><b>Debian changelog:</b></u><font size=\"-1\"><br>";
+	text += changelog;
 	text += "</font>";
-	*/
 
-	ui->label_header2->hide();
-	ui->label_header2->setText("");
 	ui->label_text->setText(text);
 }
 
@@ -383,48 +427,146 @@ void PackageSelector::on_radioTabDepends_clicked()
 	ui->label_header2->setText("");
 	ui->label_text->setText("");
 
-	QString text;
-	text += "<b>Depends:</b><br><font size=\"-1\">";
-	text += iPkg->depends().join(", ");
-	if( iPkg->depends().count()==0 )
-		text += "-";
-	text += "<br></font><br>";
+	QStringList deps = iPkg->dependsRichText();
+	QStringList confl = iPkg->conflictsRichText();
+	QStringList predeps = iPkg->preDependsRichText();
+	QStringList repl = iPkg->replacesRichText();
+	QStringList prov = iPkg->providesRichText();
+	QStringList brks = iPkg->breaksRichText();
 
-	text += "<b>Conflicts:</b><br><font size=\"-1\">";
-	text += iPkg->conflicts().join(", ");
-	if( iPkg->conflicts().count()==0)
-		text += "-";
-	text += "</font>";
+	QString text;
+
+	if( deps.count()>0 )
+	{
+		text += "<b><u>Depends:</u></b><br><font size=\"-1\">";
+		text += deps.join(", ");
+		text += "</font><br>";
+	}
+
+	if( predeps.count()>0 )
+	{
+		text += "<b><u>Pre-Depends:</u></b><br><font size=\"-1\">";
+		text += predeps.join(", ");
+		text += "</font><br>";
+	}
+
+	if( confl.count()>0 )
+	{
+		text += "<b><u>Conflicts:</u></b><br><font size=\"-1\">";
+		text += confl.join(", ");
+		text += "</font><br>";
+	}
+
+	if( prov.count()>0 )
+	{
+		text += "<b><u>Provides:</u></b><br><font size=\"-1\">";
+		text += prov.join(", ");
+		text += "</font><br>";
+	}
+
+	if( repl.count()>0 )
+	{
+		text += "<b><u>Replaces:</u></b><br><font size=\"-1\">";
+		text += repl.join(", ");
+		text += "</font><br>";
+	}
+
+	if( brks.count()>0 )
+	{
+		text += "<b><u>Breaks:</u></b><br><font size=\"-1\">";
+		text += brks.join(", ");
+		text += "</font><br>";
+	}
+
+	if( deps.count()==0 && predeps.count()==0 && confl.count()==0 && prov.count()==0 &&
+		repl.count()==0 && brks.count()==0 )
+	{
+		text = "Package has no dependencies listed";
+	}
 
 	ui->label_text->setText(text);
 }
 
-QString PackageSelector::queryChangelog()
+void PackageSelector::queryChangelog()
 {
-	QUrl url( getMaemoOrgUrl(iPkg) );
-	QNetworkAccessManager nam(this);
-	QNetworkRequest req(url);
+	if( iChangelogFetched )
+		return;
 
-	QNetworkReply* reply = nam.get(req);
+	iChangelogFetched = false;
+	iChangelog = "";
+
+	Package* pkg = iPkg->availablePackage();
+	if( !pkg )
+		pkg = iPkg;
+
+	if( getMaemoOrgUrl(pkg) == "" ) {
+		iChangelogFetched = true;
+		return;
+	}
+
+	if( !iNetworkAccessManager ) {
+		iNetworkAccessManager = new QNetworkAccessManager(this);
+		connect(iNetworkAccessManager,SIGNAL(finished(QNetworkReply*)),this,SLOT(changelogFetchNetworkReply(QNetworkReply*)));
+	}
+
+	QUrl url( getMaemoOrgUrl(pkg) );
+	QNetworkRequest req(url);
 
 	qDebug() << "fetching changelog";
 
-	while(!reply->isFinished()) {
-		QApplication::processEvents(QEventLoop::WaitForMoreEvents);
-		//if(  )
-		//reply->abort();
+	iNetworkAccessManager->get(req);
+}
+
+void PackageSelector::changelogFetchNetworkReply(QNetworkReply* reply)
+{
+	if( reply->error() == QNetworkReply::NoError ) {
+		QByteArray data = reply->readAll();
+
+		int pos = data.indexOf("<div class=\"changelog\">");
+		int pos2 = data.indexOf("</div>", pos);
+
+		if( pos!=-1 && pos2!=-1 ) {
+			iChangelog = parseChangeTable( data.mid(pos,pos2-pos+6) );
+		} else {
+			iChangelog = "";
+		}
+
+		iChangelogFetched = true;
+		qDebug() << "changelog fetched";
+	} else {
+		qDebug() << "changelog fetch error:" << reply->error() << reply->errorString();
 	}
+
+	reply->deleteLater();
+
+	if( this->isVisible() && ui->radioTabChanges->isChecked() )
+		on_radioTabChanges_clicked();
+}
+
+QString PackageSelector::parseChangeTable(QString tabledata)
+{
+	tabledata.remove("<strong>");
+	tabledata.remove("</strong>");
+	tabledata.remove('\n');
+
+	//qDebug() << tabledata;
+
+	QString newdata = "<table><tbody>";
+
+	int pos = tabledata.indexOf("<tr>", tabledata.indexOf("<tbody>") );
+	while( pos != -1 ) {
+		int pos2 = tabledata.indexOf("</td>",pos);
+		pos2 = tabledata.indexOf("</td>",pos2+5);
+
+		if( pos2 != -1 )
+			newdata += tabledata.mid(pos, pos2-pos+5) + "</tr>";
 
-	if( reply->error() != QNetworkReply::NoError ) {
-		qDebug() << "changelog fetch error:" << reply->errorString();
-		delete reply;
-		return "";
+		pos = tabledata.indexOf("<tr>",pos+4);
 	}
 
-	QByteArray data = reply->readAll();
-	delete reply;
+	newdata += "</tbody></table>";
 
+	//qDebug() << newdata;
 
-	qDebug() << "success";
-	return "";
+	return newdata;
 }
