@@ -30,6 +30,7 @@
 #include "sortselector.h"
 #include "settings.h"
 #include "searchoptions.h"
+#include "blacklistselect.h"
 
 PackageListWidgetItem::PackageListWidgetItem(Package* p_, QString name_) : QListWidgetItem(name_)
 {
@@ -51,6 +52,7 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 	int statfilter = index.data(UserRoleCurrentStatFilter).toInt();
 	int catfilter = index.data(UserRoleCurrentCatFilter).toInt();
 	QString upg_version = index.data(UserRoleAvailVersion).toString();
+	BlacklistSelect::blackList blacklisted = (BlacklistSelect::blackList) index.data(UserRoleBlacklisted).toInt();
 
 	painter->save();
 	QRect r = option.rect;
@@ -119,8 +121,19 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 	{
 		r = option.rect;
 		r.setRight( r.right()-statusicon.width()-4 );
-		painter->drawText(r, Qt::AlignTop|Qt::AlignRight, showVer, &r);
-		ver_w = r.width();
+
+		if( catfilter != PackageView::CatFilterBlacklisted ) {
+			painter->drawText(r, Qt::AlignTop|Qt::AlignRight, showVer, &r);
+			ver_w = r.width();
+		} else if( blacklisted==BlacklistSelect::BlacklistThis ) {
+			if( upgradeable && upg_version!="" ) {
+				showVer = upg_version;
+			} else {
+				showVer = version;
+			}
+			painter->drawText(r, Qt::AlignTop|Qt::AlignRight, showVer, &r);
+			ver_w = r.width();
+		}
 	}
 
 	r = option.rect;
@@ -241,9 +254,9 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 
 	iSortNoticeShown = false;
 
-	iSearchPkgName = false;
+	iSearchPkgName = true;
 	iSearchDisplayName = true;
-	iSearchDescShort = false;
+	iSearchDescShort = true;
 	iSearchDescLong = false;
 
 	// fine-tune kinetic scrolling parameters
@@ -283,13 +296,19 @@ bool PackageView::doFilterCategory(Package* pkg)
 	if( pkg->section()=="user/hidden" )
 		return false;
 
-	if( pkg->isBlacklisted() )
+	Package* upg_pkg = pkg->availablePackage();
+
+	if( iSelectedStatFilter==Package::PkgStatUpgradeable && iSelectedCatFilter!=CatFilterBlacklisted &&
+		pkg->isUpgradeable() && upg_pkg && upg_pkg->isBlacklisted() )
+		return false;
+
+	if( pkg->isBlacklisted() || (upg_pkg && upg_pkg->isBlacklisted()) )
 	{
 		if( iSelectedCatFilter == CatFilterBlacklisted )
 			return true;
 		if( pkg->isInstalled() && (iSelectedStatFilter==Package::PkgStatInstalled || iSelectedStatFilter==Package::PkgStatUnknown) )
 			return true;
-		if( iSelectedCatFilter != CatFilterBlacklisted )
+		if( pkg->isBlacklisted() && iSelectedCatFilter != CatFilterBlacklisted )
 			return false;
 	} else if( !pkg->isBlacklisted() && iSelectedCatFilter == CatFilterBlacklisted )
 		return false;
@@ -345,6 +364,15 @@ void PackageView::openWin()
 	ui->listWidget->setSortingEnabled(true);
 	iPackagesEmptySort = 0;
 
+	if( !isVisible() ) {
+		iMainWindow->busyDialog(false);
+		iListCoverLabel->setText("Loading...");
+		iListCoverLabel->setAutoFillBackground(true);
+		iListCoverLabel->show();
+		show();
+		QApplication::processEvents();
+	}
+
 	if( iSortOrder==SortDateDesc || iSortOrder==SortSizeDesc )
 		ui->listWidget->sortItems(Qt::DescendingOrder);
 	else
@@ -377,7 +405,7 @@ void PackageView::openWin()
 			while (i.hasNext())
 			{
 				i.next();
-				if( iSelectedStatFilter == Package::PkgStatUpgradeable ) {
+				if( iSelectedStatFilter == Package::PkgStatUpgradeable && iSelectedCatFilter != CatFilterBlacklisted ) {
 					if( i.value()->isUpgradeable() && doFilterCategory(i.value()) )
 						addListItem(i.value(), generateSortString(i.value()));
 				} else {
@@ -472,9 +500,14 @@ void PackageView::addListItem(Package* pkg_, QString listname_)
 		p->setData(UserRoleInstalled, pkg_->isInstalled());
 		p->setData(UserRoleUpgradeable, pkg_->isUpgradeable());
 		p->setData(UserRoleAvailVersion, pkg_->upgradeableVersion());
-		p->setData(UserRoleCurrentStatFilter, (int)iSelectedStatFilter);
+		p->setData(UserRoleCurrentStatFilter, iSelectedStatFilter);
 		p->setData(UserRoleCurrentCatFilter, iSelectedCatFilter);
 
+		if( pkg_->availablePackage() )
+			p->setData(UserRoleBlacklisted, (int)pkg_->availablePackage()->blacklisted());
+		else
+			p->setData(UserRoleBlacklisted, (int)pkg_->blacklisted());
+
 		//qDebug()<<pkg_->name();
 
 		pkg_->convertIcon();
@@ -604,6 +637,11 @@ void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 		return;
 
 	bool bl = pkg->isBlacklisted();
+	bool bl_u = false;
+
+	Package* upg_pkg = pkg->availablePackage();
+	if( upg_pkg )
+		bl_u = upg_pkg->isBlacklisted();
 
 	PackageSelector s(pkg, iAptInterface, this);
 	s.exec();
@@ -615,6 +653,9 @@ void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
 	if( pkg->isBlacklisted() != bl ) {
 		openWin();
 	}
+	else if( upg_pkg && upg_pkg->isBlacklisted() != bl_u ) {
+		openWin();
+	}
 }
 
 void PackageView::updateLabel()
@@ -1091,10 +1132,17 @@ void PackageView::on_actionRestore_all_triggered()
 	for( int i=0; i<ui->listWidget->count(); i++ )
 	{
 		Package* pkg = dynamic_cast<PackageListWidgetItem*>(ui->listWidget->item(i))->package();
-		if( pkg && pkg->isBlacklisted() ) {
-			BlacklistSelect::blackList old = pkg->blacklisted();
-			pkg->setBlacklisted(BlacklistSelect::BlacklistNone);
-			iAptInterface->removeFromBlacklist(pkg, old);
+		Package* p1 = iAptInterface->packagesInstalled()->value(pkg->name(),0);
+		Package* p2 = iAptInterface->packagesAvailable()->value(pkg->name(),0);
+		if( p1 && p1->isBlacklisted() ) {
+			BlacklistSelect::blackList old = p1->blacklisted();
+			p1->setBlacklisted(BlacklistSelect::BlacklistNone);
+			iAptInterface->removeFromBlacklist(p1, old);
+		}
+		if( p2 && p2->isBlacklisted() ) {
+			BlacklistSelect::blackList old = p2->blacklisted();
+			p2->setBlacklisted(BlacklistSelect::BlacklistNone);
+			iAptInterface->removeFromBlacklist(p2, old);
 		}
 	}
 	iAptInterface->writeBlacklist();
