@@ -37,6 +37,7 @@ RepoEdit::RepoEdit(AAptInterface* apt, Repository* repo, int pos, QWidget *paren
 	ui->lineEdit_components->setText( iRepo->components() );
 	ui->lineEdit_dist->setText( iRepo->dist() );
 	ui->checkBox_Enabled->setChecked( iRepo->enabled() );
+	iRevertEnabledTo = iRepo->enabled();
 
 	on_lineEdit_name_textEdited("");
 }
@@ -90,20 +91,29 @@ void RepoEdit::on_lineEdit_name_textEdited(QString )
 
 	if( n == "" || u == "" ) {
 		ui->btn_OK->setEnabled(false);
+		ui->checkBox_Enabled->setEnabled(false);
 		ui->checkBox_Enabled->setChecked(false);
 		return;
 	}
 
 	if( !u.contains("://") ) {
 		ui->btn_OK->setEnabled(false);
+		ui->checkBox_Enabled->setEnabled(false);
 		ui->checkBox_Enabled->setChecked(false);
 		return;
 	}
 
 	ui->btn_OK->setEnabled(true);
+	ui->checkBox_Enabled->setEnabled(true);
+	ui->checkBox_Enabled->setChecked(iRevertEnabledTo);
 }
 
 void RepoEdit::on_lineEdit_url_textEdited(QString )
 {
 	on_lineEdit_name_textEdited("");
 }
+
+void RepoEdit::on_checkBox_Enabled_clicked()
+{
+	iRevertEnabledTo = ui->checkBox_Enabled->isChecked();
+}
